@@ -1,14 +1,11 @@
+use std::future::Future;
 use std::pin::Pin;
 use std::sync::Mutex;
 
-use crate::future::{self, Future};
+use crate::future;
 use crate::io::{self, Read};
 use crate::task::{spawn_blocking, Context, JoinHandle, Poll};
-
-cfg_unstable! {
-    use once_cell::sync::Lazy;
-    use std::io::Read as _;
-}
+use crate::utils::Context as _;
 
 /// Constructs a new handle to the standard input of the current process.
 ///
@@ -59,21 +56,6 @@ pub fn stdin() -> Stdin {
 #[derive(Debug)]
 pub struct Stdin(Mutex<State>);
 
-/// A locked reference to the Stdin handle.
-///
-/// This handle implements the [`Read`] traits, and is constructed via the [`Stdin::lock`] method.
-///
-/// [`Read`]: trait.Read.html
-/// [`Stdin::lock`]: struct.Stdin.html#method.lock
-#[cfg_attr(feature = "docs", doc(cfg(unstable)))]
-#[cfg(feature = "unstable")]
-#[derive(Debug)]
-pub struct StdinLock<'a>(std::io::StdinLock<'a>);
-
-#[cfg(feature = "unstable")]
-#[cfg_attr(feature = "docs", doc(cfg(unstable)))]
-unsafe impl Send for StdinLock<'_> {}
-
 /// The state of the asynchronous stdin.
 ///
 /// The stdin can be either idle or busy performing an asynchronous operation.
@@ -161,35 +143,7 @@ impl Stdin {
             }
         })
         .await
-    }
-
-    /// Locks this handle to the standard input stream, returning a readable guard.
-    ///
-    /// The lock is released when the returned lock goes out of scope. The returned guard also implements the Read trait for accessing the underlying data.
-    ///
-    /// # Examples
-    ///
-    /// ```no_run
-    /// # fn main() -> std::io::Result<()> { async_std::task::block_on(async {
-    /// #
-    /// use async_std::io;
-    /// use async_std::prelude::*;
-    ///
-    /// let mut buffer = String::new();
-    ///
-    /// let stdin = io::stdin();
-    /// let mut handle = stdin.lock().await;
-    ///
-    /// handle.read_to_string(&mut buffer).await?;
-    /// #
-    /// # Ok(()) }) }
-    /// ```
-    #[cfg_attr(feature = "docs", doc(cfg(unstable)))]
-    #[cfg(any(feature = "unstable", feature = "docs"))]
-    pub async fn lock(&self) -> StdinLock<'static> {
-        static STDIN: Lazy<std::io::Stdin> = Lazy::new(std::io::stdin);
-
-        spawn_blocking(move || StdinLock(STDIN.lock())).await
+        .context(|| String::from("could not read line on stdin"))
     }
 }
 
@@ -262,15 +216,3 @@ cfg_windows! {
         }
     }
 }
-
-#[cfg(feature = "unstable")]
-#[cfg_attr(feature = "docs", doc(cfg(unstable)))]
-impl Read for StdinLock<'_> {
-    fn poll_read(
-        mut self: Pin<&mut Self>,
-        _cx: &mut Context<'_>,
-        buf: &mut [u8],
-    ) -> Poll<io::Result<usize>> {
-        Poll::Ready(self.0.read(buf))
-    }
-}
